@@ -190,7 +190,7 @@ static PHP_METHOD(UTCDateTime, __construct)
 	}
 
 	if (Z_TYPE_P(milliseconds) != IS_STRING) {
-		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Expected integer or string, %s given", zend_get_type_by_const(Z_TYPE_P(milliseconds)));
+		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Expected integer or string, %s given", PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(milliseconds));
 		return;
 	}
 
@@ -396,7 +396,7 @@ static PHP_METHOD(UTCDateTime, unserialize)
 } /* }}} */
 
 /* {{{ MongoDB\BSON\UTCDateTime function entries */
-ZEND_BEGIN_ARG_INFO_EX(ai_UTCDateTime___construct, 0, 0, 1)
+ZEND_BEGIN_ARG_INFO_EX(ai_UTCDateTime___construct, 0, 0, 0)
 	ZEND_ARG_INFO(0, milliseconds)
 ZEND_END_ARG_INFO()
 
