@@ -47,7 +47,7 @@ static bool php_phongo_bson_visit_array(const bson_iter_t *iter ARG_UNUSED, cons
 
 static void php_phongo_bson_visit_corrupt(const bson_iter_t *iter ARG_UNUSED, void *data ARG_UNUSED) /* {{{ */
 {
-	mongoc_log(MONGOC_LOG_LEVEL_TRACE, MONGOC_LOG_DOMAIN, "Corrupt BSON data detected!");
+	mongoc_log(MONGOC_LOG_LEVEL_WARNING, MONGOC_LOG_DOMAIN, "Corrupt BSON data detected!");
 } /* }}} */
 
 static void php_phongo_bson_visit_unsupported_type(const bson_iter_t *iter ARG_UNUSED, const char *key, uint32_t v_type_code, void *data ARG_UNUSED) /* {{{ */
@@ -131,8 +131,30 @@ static bool php_phongo_bson_visit_binary(const bson_iter_t *iter ARG_UNUSED, con
 
 static bool php_phongo_bson_visit_undefined(const bson_iter_t *iter, const char *key, void *data) /* {{{ */
 {
+	zval *retval = PHONGO_BSON_STATE_ZCHILD(data);
+#if PHP_VERSION_ID >= 70000
+	zval zchild;
+
+	object_init_ex(&zchild, php_phongo_undefined_ce);
+
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_zval(retval, &zchild);
+	} else {
+		ADD_ASSOC_ZVAL(retval, key, &zchild);
+	}
+#else
+	zval *zchild = NULL;
 	TSRMLS_FETCH();
-	phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "Detected unsupported BSON type 0x06 (undefined) for fieldname \"%s\"", key);
+
+	MAKE_STD_ZVAL(zchild);
+	object_init_ex(zchild, php_phongo_undefined_ce);
+
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_zval(retval, zchild);
+	} else {
+		ADD_ASSOC_ZVAL(retval, key, zchild);
+	}
+#endif
 
 	return false;
 } /* }}} */
@@ -283,10 +305,32 @@ static bool php_phongo_bson_visit_regex(const bson_iter_t *iter ARG_UNUSED, cons
 	return false;
 } /* }}} */
 
-static bool php_phongo_bson_visit_symbol(const bson_iter_t *iter, const char *key, size_t symbol_len, const char *symbol, void *data) /* {{{ */
+static bool php_phongo_bson_visit_symbol(const bson_iter_t *iter, const char *key, size_t v_symbol_len, const char *v_symbol, void *data) /* {{{ */
 {
+	zval *retval = PHONGO_BSON_STATE_ZCHILD(data);
+#if PHP_VERSION_ID >= 70000
+	zval zchild;
+
+	php_phongo_new_symbol(&zchild, v_symbol, v_symbol_len TSRMLS_CC);
+
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_zval(retval, &zchild);
+	} else {
+		ADD_ASSOC_ZVAL(retval, key, &zchild);
+	}
+#else
+	zval *zchild = NULL;
 	TSRMLS_FETCH();
-	phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "Detected unsupported BSON type 0x0E (symbol) for fieldname \"%s\"", key);
+
+	MAKE_STD_ZVAL(zchild);
+	php_phongo_new_symbol(zchild, v_symbol, v_symbol_len TSRMLS_CC);
+
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_zval(retval, zchild);
+	} else {
+		ADD_ASSOC_ZVAL(retval, key, zchild);
+	}
+#endif
 
 	return false;
 } /* }}} */
@@ -321,10 +365,32 @@ static bool php_phongo_bson_visit_code(const bson_iter_t *iter ARG_UNUSED, const
 	return false;
 } /* }}} */
 
-static bool php_phongo_bson_visit_dbpointer(const bson_iter_t *iter, const char *key, size_t collection_len, const char *collection, const bson_oid_t *oid, void *data) /* {{{ */
+static bool php_phongo_bson_visit_dbpointer(const bson_iter_t *iter, const char *key, size_t namespace_len, const char *namespace, const bson_oid_t *oid, void *data) /* {{{ */
 {
+	zval *retval = PHONGO_BSON_STATE_ZCHILD(data);
+#if PHP_VERSION_ID >= 70000
+	zval zchild;
+
+	php_phongo_new_dbpointer(&zchild, namespace, namespace_len, oid TSRMLS_CC);
+
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_zval(retval, &zchild);
+	} else {
+		ADD_ASSOC_ZVAL(retval, key, &zchild);
+	}
+#else
+	zval *zchild = NULL;
 	TSRMLS_FETCH();
-	phongo_throw_exception(PHONGO_ERROR_UNEXPECTED_VALUE TSRMLS_CC, "Detected unsupported BSON type 0x0C (DBPointer) for fieldname \"%s\"", key);
+
+	MAKE_STD_ZVAL(zchild);
+	php_phongo_new_dbpointer(zchild, namespace, namespace_len, oid TSRMLS_CC);
+
+	if (((php_phongo_bson_state *)data)->is_visiting_array) {
+		add_next_index_zval(retval, zchild);
+	} else {
+		ADD_ASSOC_ZVAL(retval, key, zchild);
+	}
+#endif
 
 	return false;
 } /* }}} */
